function resultAvg = computeAverage(result)

    mesh = {result(:).mesh};
    includedNodes = [result(:).includedNodes];

    % Consider the case where meshes are not the same
    % Some nodes may be missing due to significant wrinkling
    if ~all(includedNodes,'all')
        [mesh,usedNodes,usedElements] = combineDissimilarMeshes(mesh,includedNodes);

        for i=1:numel(result)
            result(i).includedNodes(~usedNodes) = false;
            fields = fieldnames(result(i).elements);
            for j=1:numel(fields)
                result(i).elements.(fields{j}) = result(i).elements.(fields{j})(usedElements{i},:);
            end
            fields = fieldnames(result(i).nodes);
            for j=1:numel(fields)
                selection = usedNodes(includedNodes(:,i));
                result(i).nodes.(fields{j}) = result(i).nodes.(fields{j})(selection,:);
            end
        end
    end

    % Assemble the results
    % The meshes from all the individual results are included. 
    % Individual results are "stacked" in the 3-dimension for easier
    % computation of the mean and standard deviation in the next step
    resultAvg = result(1);
    resultAvg.mesh = mesh;
    resultAvg.meshRef = {result(:).meshRef};
    for i=2:numel(result)
        fields = fieldnames(result(i).elements);
        for j=1:numel(fields)
            resultAvg.elements.(fields{j}) = cat(3,resultAvg.elements.(fields{j}),result(i).elements.(fields{j}));
        end
        fields = fieldnames(result(i).nodes);
        for j=1:numel(fields)
            resultAvg.nodes.(fields{j}) = cat(3,resultAvg.nodes.(fields{j}),result(i).nodes.(fields{j}));
        end
    end

    % Compute mean and std
    % Perform all calculations in the 3-dimension
    fields = fieldnames(resultAvg.elements);
    for j=1:numel(fields)
        resultAvg.elements_std.(fields{j}) = std(resultAvg.elements.(fields{j}),[],3);
        resultAvg.elements.(fields{j}) = mean(resultAvg.elements.(fields{j}),3);
    end
    fields = fieldnames(resultAvg.nodes);
    for j=1:numel(fields)
        resultAvg.nodes_std.(fields{j}) = std(resultAvg.nodes.(fields{j}),[],3);
        resultAvg.nodes.(fields{j}) = mean(resultAvg.nodes.(fields{j}),3);
    end

end

function [meshOut,usedNodes,usedElements] = combineDissimilarMeshes(meshIn,includedNodes)

    % Only include nodes that are present in all three meshes
    usedNodes = all(includedNodes,2);
    % Exclude nodes not attached to any element
    includedNodes(includedNodes(:,1),4) = floatingNodeCorrection(meshIn{1},usedNodes(includedNodes(:,1)));
    usedNodes = all(includedNodes,2);

    % Update the meshes
    for i=1:numel(meshIn)
        [meshOut{i},usedElements{i}] = meshReduce(meshIn{i},usedNodes(includedNodes(:,i)));
    end

end

function [selection] = floatingNodeCorrection(mesh,selection)
    % Filter elements based on selected nodes
    elements = mesh.ConnectivityList;
    elementSelect = all(reshape(selection(elements),size(elements)),2);
    elements = elements(elementSelect,:);

    % Find nodes that are no longer referenced by the mesh
    floatingNodes = ~ismember(find(selection),unique(elements));

    % Only use non-floating nodes
    selection(selection) = ~floatingNodes;
end

function [mesh,elementSelect] = meshReduce(mesh,selection)
    
    % Only include the nodes in the selection
    nodes = mesh.Points(selection,:);
    % Keep the relation between old and new numbers in an index
    index(find(selection)) = 1:sum(selection);

    % Select only elements that have all nodes in the selection
    elements = mesh.ConnectivityList;
    elementSelect = reshape(selection(elements),size(elements));
    elementSelect = all(elementSelect,2);
    
    % Only keep elements selected, then renumber using the index
    elements = elements(elementSelect,:);
    elements = reshape(index(elements),size(elements));

    % Assemble the output variable
    mesh = triangulation(elements,nodes);

end