function [result] = computeElementDeformation(undeformed,deformed,localcs1D,fiber1,fiber2)

    % Check the input
    if ~all(size(undeformed.Points)==size(deformed.Points))
        error('Number of nodes must match')
    end
    if ~all(size(undeformed.ConnectivityList),size(deformed.ConnectivityList))
        error('Number of elements must match')
    end
    if ~all(undeformed.ConnectivityList==deformed.ConnectivityList,'all')
        error('Element definitions must be exactly the same between meshes')
    end
    if numel(localcs1D)~=3 || numel(fiber1)~=2 || numel(fiber2)~=2
        error('Fiber definitions are not the correct dimension')
    end

    % Make sure all are column vectors
    localcs1D = localcs1D(:);
    fiber1 = fiber1(:);
    fiber2 = fiber2(:);

    % pre-allocate results
    numE = size(undeformed.ConnectivityList,1);
    result.F = cell(numE,1);
    result.GLstrainXX = zeros(numE,1);
    result.GLstrainYY = zeros(numE,1);
    result.GLstrainXY = zeros(numE,1);
    result.shearangle = zeros(numE,1);
    result.fiber1 = zeros(numE,3);
    result.fiber2 = zeros(numE,3);

    % Loop over all elements
    for i = 1:numE
        % Setup calculations based on the undeformed element
        X = undeformed.Points(undeformed.ConnectivityList(i,:),:);
        lcsX = getLCS(X);
        Q = getTransformation(lcsX,localcs1D);
        fiber1X = Q'*fiber1;
        fiber2X = Q'*fiber2;

        % Continue calculation on the deformed element
        x = deformed.Points(deformed.ConnectivityList(i,:),:);
        lcsx = getLCS(x);
        F = calcF_3D(X,x);
        E = 0.5*(F'*F-eye(2));
        E = Q*E*Q';
        fiber1x = F*fiber1X;
        fiber1x = fiber1x/norm(fiber1x);
        fiber2x = F*fiber2X;
        fiber2x = fiber2x/norm(fiber2x);

        % Assign results to output
        result.F{i} = F;
        result.GLstrainXX(i) = E(1,1);
        result.GLstrainYY(i) = E(2,2);
        result.GLstrainXY(i) = E(1,2);
        result.shearangle(i) = acosd(fiber1x'*fiber2x)-acosd(fiber1X'*fiber2X);
        result.fiber1(i,:) = lcsx*[fiber1x;0];
        result.fiber2(i,:) = lcsx*[fiber2x;0];
    end

end

function lcs = getLCS(X)
    % Compute orthonormal element coordinate system for a triangular element
    % 1-dir is between node1 and node2
    % 3-dir is perpendicular to surface. Right-hand rule determines direction.
    % 2-dir is normal to the 1-3 plane. Right-hand rule determines direction.
    X = X';
    v1 = X(:,2)-X(:,1);
    v2 = X(:,3)-X(:,1);
    lcs3 = cross(v1,v2);
    lcs3 = lcs3/norm(lcs3);
    lcs1 = v1;
    lcs1 = lcs1/norm(lcs1);
    lcs2 = cross(lcs3,lcs1);
    lcs = [lcs1 lcs2 lcs3];
end

function Q = getTransformation(lcs,dir)
    % Calculates the transformation matrix between element coordinates and
    % local coordinates.
    dir = dir/norm(dir);
    dir = lcs'*dir;
    dir = dir(1:2);
    dir = dir/norm(dir);
    Q = [dir(1) -dir(2);dir(2) dir(1)]';
end

function X = triangle3Dto2D(X)
    % Transforms 3D node coordinates to 2D in element coordinate system
    lcs = getLCS(X);
    X = X-repmat(X(1,:),3,1);
    X = X*lcs;
    X = X(:,[1 2]);
end

function F = calcF_3D(X,x)
    % Calculates deformation gradient F between two triangles in 3D
    X = triangle3Dto2D(X);
    x = triangle3Dto2D(x);
    u = x-X;
    F = calcF_2D(X,u);
end

function F = calcF_2D(X,u)
    % Calculates deformation gradient F between two triangles in 2D
%     F = eye(2) + [u(1,1)-u(3,1) u(2,1)-u(3,1);u(1,2)-u(3,2) u(2,2)-u(3,2)]*inv([X(1,1)-X(3,1) X(2,1)-X(3,1);X(1,2)-X(3,2) X(2,2)-X(3,2)]);
    F = eye(2) + [u(1,1)-u(3,1) u(2,1)-u(3,1);u(1,2)-u(3,2) u(2,2)-u(3,2)]/[X(1,1)-X(3,1) X(2,1)-X(3,1);X(1,2)-X(3,2) X(2,2)-X(3,2)];
end