clear all;close all;clc

addpath('code')
addpath('code_deformation')

colorScale = [0 0.15];
colorLabel = '$|$Eng. shear strain$|$ [rad]';

specimenNames = {'2T','4T','10T'};

load('referenceTorsionMesh.mat','referenceTorsionMesh','referenceTorsionMeshUnfold')
doUnfold = true; % Optional. Set to false for 3D mesh

%% Theoretical deformation distribution
X2 = referenceTorsionMesh.Points(:,1);
X3 = referenceTorsionMesh.Points(:,2);
[gamma12,gamma13] = freeTorsionStrainsIsotropic(X2,X3,10,10,30/180*pi/48,20);

gamma = max(abs(gamma12),abs(gamma13));
newDeformationPlot(referenceTorsionMesh,gamma,'Isotropic',colorScale,colorLabel,doUnfold);
exportgraphics(gcf,'deformationTheoretical.pdf','ContentType','vector')

%% Calculate average deformation and plot TC1225
materialName = 'TC';
TC1225 = getDeformation(materialName);
for i=1:3
    gamma = 2*abs(TC1225(i).nodes.GLstrainXY);
    axisTitle = append(materialName,specimenNames{i});
    newDeformationPlot(TC1225(i).mesh,gamma,axisTitle,colorScale,colorLabel,doUnfold);
    % exportgraphics(gcf,sprintf('deformationTC1225_%s.pdf',specimenNames{i}),'ContentType','vector')
end

TC1225 = averageDeformation(TC1225);
gammaTC1225 = 2*abs(TC1225.nodes.GLstrainXY);
axisTitle = append('Toray TC1225',newline,'Average');
newDeformationPlot(referenceTorsionMesh,gammaTC1225,axisTitle,colorScale,colorLabel,doUnfold);
% exportgraphics(gcf,'deformationTC1225_average.pdf','ContentType','vector')

%% Fit anisotropic distribution TC1225
f_TC1225 = fitAnisotropy(referenceTorsionMesh,gammaTC1225);
[gamma12,gamma13] = freeTorsionStrainsAnisotropic(X2,X3,10,10,30/180*pi/48,20,f_TC1225);
shearModel = max(abs(gamma12),abs(gamma13));
axisTitle = append('Toray TC1225',newline,sprintf('f = %1.3g',f_TC1225));
newDeformationPlot(referenceTorsionMesh,shearModel,axisTitle,colorScale,colorLabel,doUnfold);
% exportgraphics(gcf,'deformationTC1225_fitAniso.pdf','ContentType','vector')

%% Calculate average deformation and plot APC
materialName = 'APC';
APC = getDeformation(materialName);
for i=1:3
    gamma = 2*abs(APC(i).nodes.GLstrainXY);
    axisTitle = append(materialName,specimenNames{i});
    newDeformationPlot(APC(i).mesh,gamma,axisTitle,colorScale,colorLabel,doUnfold);
    % exportgraphics(gcf,sprintf('deformationAPC_%s.pdf',specimenNames{i}),'ContentType','vector')
end

APC = averageDeformation(APC);
gammaAPC = 2*abs(APC.nodes.GLstrainXY);
axisTitle = append('Solvay APC',newline,'Average');
newDeformationPlot(referenceTorsionMesh,gammaAPC,axisTitle,colorScale,colorLabel,doUnfold);
% exportgraphics(gcf,'deformationAPC_average.pdf','ContentType','vector')

%% Fit anisotropic distribution APC
f_APC = fitAnisotropy(referenceTorsionMesh,gammaAPC);
[gamma12,gamma13] = freeTorsionStrainsAnisotropic(X2,X3,10,10,30/180*pi/48,20,f_APC);
shearModel = max(abs(gamma12),abs(gamma13));
axisTitle = append('Solvay APC',newline,sprintf('f = %1.3g',f_APC));
newDeformationPlot(referenceTorsionMesh,shearModel,axisTitle,colorScale,colorLabel,doUnfold);
% exportgraphics(gcf,'deformationAPCfitAniso.pdf','ContentType','vector')

%% FUNCTIONS 
function result = getDeformation(materialName)
    localCS1D = [0 0 1];
    fiber1 = [1 0];
    fiber2 = [0 1];
    specimenName = strcat(materialName,{'2T','4T','10T'});
    for i=1:numel(specimenName)
        undeformedFile = fullfile('rawDataDeformation',[specimenName{i} '_undeformed.msh']);
        deformedFile = fullfile('rawDataDeformation',[specimenName{i} '_deformed.msh']);
        stateUndeformed = loadDeformationState(undeformedFile);
        stateDeformed = loadDeformationState(deformedFile); 
        result(i) = computeDeformation(stateUndeformed,stateDeformed,localCS1D,fiber1,fiber2);
        result(i).elements = rmfield(result(i).elements,'F');
    end
end

function average = averageDeformation(result)
    average = computeAverage(result);
    average = rmfield(average,'nodes');
    average.nodes = extrapolateToNodes(average.elements,average.mesh{1});
end

function newDeformationPlot(mesh,result,axisTitle,colorScale,colorLabel,doUnfold)
    if doUnfold
        load('referenceTorsionMesh.mat','referenceTorsionMeshUnfold')
        mesh = referenceTorsionMeshUnfold;
    end
    figure('color','w','Units','centimeters','Position',[10 10 10.5 8]);
    plotMesh(mesh,result);
    title(gca,axisTitle)
    if doUnfold
        unfoldedPlotAnnotation()
    end
    plotColorbar(colorScale,colorLabel);
    fixPlotText()
end

function unfoldedPlotAnnotation()
    hold on
    text([-10 0 10 20],ones(1,4)*23.5,{'left','front','right','back'},'HorizontalAlignment','center')
    text([-10 0 10 20],ones(1,4)*-23.5,{'$\gamma_{13}$','$\gamma_{12}$','$\gamma_{13}$','$\gamma_{12}$'},'HorizontalAlignment','center','Interpreter','latex')
    view(0,90)
    a = gca;
    a.Title.Position(2) = 26;
end

function f = fitAnisotropy(mesh,shearExperimental)
    X2 = mesh.Points(:,1);
    X3 = mesh.Points(:,2);
    f = fminsearch(@(x) objectiveAnisotropy(x,X2,X3,shearExperimental),1);   
end

function goal = objectiveAnisotropy(x,X2,X3,shearExperimental)
    f = x(1);
    [gamma12,gamma13] = freeTorsionStrainsAnisotropic(X2,X3,10,10,30/180*pi/48,20,f);
    shearModel = max(abs(gamma12),abs(gamma13));
    goal = sum((shearModel-shearExperimental).^2);
end
